@@ -1,24 +1,29 @@
 """Provide strategies for given endpoint(s) definition."""
 import asyncio
-import inspect
 import re
-from base64 import b64encode
 from functools import partial
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Union
 from urllib.parse import quote_plus
 
 import hypothesis
-import hypothesis.strategies as st
+from hypothesis import strategies as st
 from hypothesis_jsonschema import from_schema
-from requests.auth import _basic_auth_str
 
 from . import utils
 from .exceptions import InvalidSchema
 from .hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher
 from .models import Case, Endpoint
-from .types import Hook
+from .specs.openapi._hypothesis import STRING_FORMATS
 
 PARAMETERS = frozenset(("path_parameters", "headers", "cookies", "query", "body", "form_data"))
+LOCATION_TO_CONTAINER = {
+    "path": "path_parameters",
+    "query": "query",
+    "header": "headers",
+    "cookie": "cookies",
+    "body": "body",
+    "formData": "form_data",
+}
 SLASH = "/"
 
 
@@ -31,9 +36,8 @@ def create_test(
     wrapped_test = hypothesis.given(case=strategy)(test)
     if seed is not None:
         wrapped_test = hypothesis.seed(seed)(wrapped_test)
-    original_test = get_original_test(test)
-    if asyncio.iscoroutinefunction(original_test):
-        wrapped_test.hypothesis.inner_test = make_async_test(original_test)  # type: ignore
+    if asyncio.iscoroutinefunction(test):
+        wrapped_test.hypothesis.inner_test = make_async_test(test)  # type: ignore
     if settings is not None:
         wrapped_test = settings(wrapped_test)
     return add_examples(wrapped_test, endpoint, hook_dispatcher=hook_dispatcher)
@@ -48,19 +52,6 @@ def make_test_or_exception(
         return exc
 
 
-def get_original_test(test: Callable) -> Callable:
-    """Get the original test function even if it is wrapped by `hypothesis.settings` decorator.
-
-    Applies only to Hypothesis pre 4.42.4 versions.
-    """
-    # `settings` decorator is applied
-    if getattr(test, "_hypothesis_internal_settings_applied", False) and hypothesis.__version_info__ < (4, 42, 4):
-        # This behavior was changed due to a bug - https://github.com/HypothesisWorks/hypothesis/issues/2160
-        # And since Hypothesis 4.42.4 is no longer required
-        return test._hypothesis_internal_test_function_without_warning  # type: ignore
-    return test
-
-
 def make_async_test(test: Callable) -> Callable:
     def async_run(*args: Any, **kwargs: Any) -> None:
         loop = asyncio.get_event_loop()
@@ -71,29 +62,9 @@ def make_async_test(test: Callable) -> Callable:
     return async_run
 
 
-def get_example(endpoint: Endpoint) -> Optional[st.SearchStrategy[Case]]:
-    static_parameters = {}
-    for name in PARAMETERS:
-        parameter = getattr(endpoint, name)
-        if parameter is not None and "example" in parameter:
-            static_parameters[name] = parameter["example"]
-    if static_parameters:
-        strategies = {
-            parameter: prepare_strategy(parameter, getattr(endpoint, parameter))
-            for parameter in PARAMETERS - set(static_parameters)
-            if getattr(endpoint, parameter) is not None
-        }
-        return _get_case_strategy(endpoint, static_parameters, strategies)
-    return None
-
-
 def add_examples(test: Callable, endpoint: Endpoint, hook_dispatcher: Optional[HookDispatcher] = None) -> Callable:
     """Add examples to the Hypothesis test, if they are specified in the schema."""
-    strategy = get_example(endpoint)
-    if strategy is not None:
-        examples = get_single_example(strategy)
-    else:
-        examples = []
+    examples: List[Case] = [get_single_example(strategy) for strategy in endpoint.get_strategies_from_examples()]
     context = HookContext(endpoint)  # context should be passed here instead
     GLOBAL_HOOK_DISPATCHER.dispatch("before_add_examples", context, examples)
     endpoint.schema.hooks.dispatch("before_add_examples", context, examples)
@@ -104,7 +75,7 @@ def add_examples(test: Callable, endpoint: Endpoint, hook_dispatcher: Optional[H
     return test
 
 
-def get_single_example(strategy: st.SearchStrategy[Case]) -> List[Case]:
+def get_single_example(strategy: st.SearchStrategy[Case]) -> Case:
     @hypothesis.given(strategy)  # type: ignore
     @hypothesis.settings(  # type: ignore
         database=None,
@@ -119,12 +90,14 @@ def get_single_example(strategy: st.SearchStrategy[Case]) -> List[Case]:
 
     examples: List[Case] = []
     example_generating_inner_function()
-    return examples
+    return examples[0]
 
 
-def is_valid_header(headers: Dict[str, str]) -> bool:
+def is_valid_header(headers: Dict[str, Any]) -> bool:
     """Verify if the generated headers are valid."""
     for name, value in headers.items():
+        if not isinstance(value, str):
+            return False
         if not utils.is_latin_1_encodable(value):
             return False
         if utils.has_invalid_characters(name, value):
@@ -132,7 +105,7 @@ def is_valid_header(headers: Dict[str, str]) -> bool:
     return True
 
 
-def is_surrogate(item: Any) -> bool:
+def is_illegal_surrogate(item: Any) -> bool:
     return isinstance(item, str) and bool(re.search(r"[\ud800-\udfff]", item))
 
 
@@ -142,7 +115,7 @@ def is_valid_query(query: Dict[str, Any]) -> bool:
     `requests` and `werkzeug` will fail to send it to the application.
     """
     for name, value in query.items():
-        if is_surrogate(name) or is_surrogate(value):
+        if is_illegal_surrogate(name) or is_illegal_surrogate(value):
             return False
     return True
 
@@ -153,27 +126,48 @@ def get_case_strategy(endpoint: Endpoint, hooks: Optional[HookDispatcher] = None
     Path & endpoint are static, the others are JSON schemas.
     """
     strategies = {}
-    static_kwargs: Dict[str, Any] = {"endpoint": endpoint}
-    try:
-        for parameter in PARAMETERS:
-            value = getattr(endpoint, parameter)
-            if value is not None:
-                strategies[parameter] = prepare_strategy(parameter, value)
-            else:
-                static_kwargs[parameter] = None
-        return _get_case_strategy(endpoint, static_kwargs, strategies, hooks)
-    except AssertionError:
-        raise InvalidSchema("Invalid schema for this endpoint")
+    static_kwargs: Dict[str, Any] = {}
+    for parameter in PARAMETERS:
+        value = getattr(endpoint, parameter)
+        if value is not None:
+            location = {"headers": "header", "cookies": "cookie", "path_parameters": "path"}.get(parameter, parameter)
+            strategies[parameter] = prepare_strategy(parameter, value, endpoint.get_hypothesis_conversions(location))
+        else:
+            static_kwargs[parameter] = None
+    return _get_case_strategy(endpoint, static_kwargs, strategies, hooks)
 
 
-def prepare_strategy(parameter: str, value: Dict[str, Any]) -> st.SearchStrategy:
+def to_bytes(value: Union[str, bytes, int, bool, float]) -> bytes:
+    return str(value).encode(errors="ignore")
+
+
+def is_valid_form_data(form_data: Any) -> bool:
+    return isinstance(form_data, dict)
+
+
+def prepare_form_data(form_data: Dict[str, Any]) -> Dict[str, Any]:
+    for name, value in form_data.items():
+        if isinstance(value, list):
+            form_data[name] = [to_bytes(item) if not isinstance(item, (bytes, str, int)) else item for item in value]
+        elif not isinstance(value, (bytes, str, int)):
+            form_data[name] = to_bytes(value)
+    return form_data
+
+
+def prepare_strategy(parameter: str, value: Dict[str, Any], map_func: Optional[Callable]) -> st.SearchStrategy:
+    """Create a strategy for a schema and add location-specific filters & maps."""
+    strategy = from_schema(value, custom_formats=STRING_FORMATS)
+    if map_func is not None:
+        strategy = strategy.map(map_func)
     if parameter == "path_parameters":
-        return from_schema(value).filter(filter_path_parameters).map(quote_all)  # type: ignore
-    if parameter in ("headers", "cookies"):
-        return from_schema(value).filter(is_valid_header)  # type: ignore
-    if parameter == "query":
-        return from_schema(value).filter(is_valid_query)  # type: ignore
-    return from_schema(value)  # type: ignore
+        strategy = strategy.filter(filter_path_parameters).map(quote_all)  # type: ignore
+    elif parameter in ("headers", "cookies"):
+        strategy = strategy.filter(is_valid_header)  # type: ignore
+    elif parameter == "query":
+        strategy = strategy.filter(is_valid_query)  # type: ignore
+    elif parameter == "form_data":
+        strategy = strategy.filter(is_valid_form_data).map(prepare_form_data)  # type: ignore
+    return strategy
 
 
 def filter_path_parameters(parameters: Dict[str, Any]) -> bool:
@@ -190,12 +184,13 @@ def filter_path_parameters(parameters: Dict[str, Any]) -> bool:
     path_parameter_blacklist = (".", SLASH, "")
 
     return not any(
-        (value in path_parameter_blacklist or isinstance(value, str) and SLASH in value)
+        (value in path_parameter_blacklist or is_illegal_surrogate(value) or isinstance(value, str) and SLASH in value)
         for value in parameters.values()
     )
 
 
 def quote_all(parameters: Dict[str, Any]) -> Dict[str, Any]:
+    """Apply URL quotation for all values in a dictionary."""
     return {key: quote_plus(value) if isinstance(value, str) else value for key, value in parameters.items()}
 
 
@@ -204,7 +199,7 @@ def _get_case_strategy(
     extra_static_parameters: Dict[str, Any],
     strategies: Dict[str, st.SearchStrategy],
     hook_dispatcher: Optional[HookDispatcher] = None,
-) -> st.SearchStrategy:
+) -> st.SearchStrategy[Case]:
     static_parameters: Dict[str, Any] = {"endpoint": endpoint, **extra_static_parameters}
     if endpoint.schema.validate_schema and endpoint.method == "GET":
         if endpoint.body is not None:
@@ -224,35 +219,4 @@ def _apply_hooks(strategies: Dict[str, st.SearchStrategy], dispatcher: HookDispa
         for hook in dispatcher.get_all_by_name(f"before_generate_{key}"):
             # Get the strategy on each hook to pass the first hook output as an input to the next one
             strategy = strategies[key]
-            args: Union[Tuple[st.SearchStrategy], Tuple[HookContext, st.SearchStrategy]]
-            if _accepts_context(hook):
-                args = (context, strategy)
-            else:
-                args = (strategy,)
-            strategies[key] = hook(*args)
-
-
-def _accepts_context(hook: Hook) -> bool:
-    # There are no restrictions on the first argument's name and we don't check its name here.
-    return len(inspect.signature(hook).parameters) == 2
-
-
-def register_string_format(name: str, strategy: st.SearchStrategy) -> None:
-    if not isinstance(name, str):
-        raise TypeError(f"name must be of type {str}, not {type(name)}")
-    if not isinstance(strategy, st.SearchStrategy):
-        raise TypeError(f"strategy must be of type {st.SearchStrategy}, not {type(strategy)}")
-    from hypothesis_jsonschema._from_schema import STRING_FORMATS  # pylint: disable=import-outside-toplevel
-
-    STRING_FORMATS[name] = strategy
-
-
-def init_default_strategies() -> None:
-    register_string_format("binary", st.binary())
-    register_string_format("byte", st.binary().map(lambda x: b64encode(x).decode()))
-
-    def make_basic_auth_str(item: Tuple[str, str]) -> str:
-        return _basic_auth_str(*item)
-
-    register_string_format("_basic_auth", st.tuples(st.text(), st.text()).map(make_basic_auth_str))  # type: ignore
-    register_string_format("_bearer_auth", st.text().map("Bearer {}".format))
+            strategies[key] = hook(context, strategy)
